
####################################################################################
# This code does the species abundance models for 135 vascular plant species
# and calculates community weighted mean values from the projections

# Analyses used in paper: Decreasing snow cover alters functional composition and diversity of Arctic tundra
# By: Pekka Niittynen, Risto K. Heikkinen and Miska Luoto
# Author: Pekka Niittynen
# email: pekka.niittynen@helsinki.fi
# Modified and deposited: 2020-07-22
#

#install.packages("caret", dependencies = c("Depends", "Suggests"))
library(caret)
library(randomForest)
library(dplyr)
library(PresenceAbsence)
library(ecospat)
library(doParallel)
library(crayon)
library(vegan)
library(raster)
library(data.table)

# Set working directory
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
rasterwd <- dirname(rstudioapi::getActiveDocumentContext()$path)

# Import files
d <- fread("data/species_env_data.csv")

# Define response and explanatory variables
respvars <- names(d)[9:NCOL(d)]
cvr_sums <- rowSums(d %>% dplyr::select(respvars))
expvars <- c("temperature","scd","radiation","soil","edap","swi")
categorical <- "soil"

# chech correlations between environmental variables
cor(d %>% dplyr::select(expvars), use = "pairwise.complete.obs")

# Prevalence threshold to remove too rare species from modelling
threshold_prevalence <- 8

# Transformation
trans_method <- "No" # or "square" or "No" or "cubic"

# Response family for the GBM models
respfamily <- "gaussian"

# Use parallel processing?
multicore <- TRUE
cores <- detectCores()-1

# Seed for reproducible results
seed <- 3595

# Which modelling methods to use
models <- c("rf", "gbm")

# Metric for ensemble model construction
ens_metric <- "R2"
ensemble <- T

tag <- "CVR_models"
model_date <- Sys.Date()

# Cross-validation options
cv_method <- "repeatedcv"
cv_number <- 4
cv_repeats <- 3

# Method to standardize the cover data
# "none", "stand", "stand_by_all", "stand_100max", stand_by_all_100max
stand_method <- "stand_by_all_100max"

# Tuning parameters for GBM
tunegrid_gbm <- expand.grid(interaction.depth=c(3:5), n.trees = (1:5)*500,
                            shrinkage=c(0.01, 0.005, 0.001),
                            n.minobsinnode=c(5,10))
tunegrid_rf <- expand.grid(mtry=c(2,4,6,8,10))


# Raster predictor processing
prediction <- TRUE
only_ensemble_pred <- TRUE

if(prediction){
  pred_stack <- stack("data/raster_stack.tif")
  names(pred_stack) <- c("temperature","scd","radiation","soil","edap","swi")
  
  # CREATE FUTURE CLIMATOLOGY
  pred_stack[["temperature_26"]] <- pred_stack[["temperature"]] + ((1.805+1.792+1.680)/3)
  pred_stack[["temperature_45"]] <- pred_stack[["temperature"]] + ((2.228+2.093+2.236)/3)
  pred_stack[["temperature_85"]] <- pred_stack[["temperature"]] + ((2.876+2.894+2.891)/3)
  
  pred_stack[["scd_10add"]] <- pred_stack[["scd"]]*1.1
  pred_stack[["scd_05add"]] <- pred_stack[["scd"]]*1.05
  pred_stack[["scd_05"]] <- pred_stack[["scd"]]*0.95
  pred_stack[["scd_10"]] <- pred_stack[["scd"]]*0.9
  pred_stack[["scd_15"]] <- pred_stack[["scd"]]*0.85
  pred_stack[["scd_20"]] <- pred_stack[["scd"]]*0.8
  pred_stack[["scd_25"]] <- pred_stack[["scd"]]*0.75
  pred_stack[["scd_30"]] <- pred_stack[["scd"]]*0.7
  pred_stack[["scd_35"]] <- pred_stack[["scd"]]*0.65
  pred_stack[["scd_40"]] <- pred_stack[["scd"]]*0.6
  
  pred_mod_grid <- expand.grid(temperature = c("temperature", "temperature_26", "temperature_45", "temperature_85"),
                               scd = c("scd", "scd_10add", "scd_05add", "scd_05", "scd_10", "scd_15", "scd_20",
                                       "scd_25", "scd_30", "scd_35", "scd_40"),
                               radiation = "radiation",
                               soil = "soil",
                               edap = "edap",
                               swi = "swi", stringsAsFactors = F)
  
  #Select variable(s) used to name the different prediction datasets
  naming_vars <- c("temperature", "scd")
  modlist <- c()
  for(i in 1:NROW(pred_mod_grid)){
    modlist[[paste(pred_mod_grid[i,naming_vars], collapse = "__")]] <- as.character(pred_mod_grid[i,])
  }
  pred_grid <- as.data.frame(pred_stack)
  
  pred_grid[!complete.cases(pred_grid),] <- NA
  
  # Create empty raster template for the predictions
  templ_raster <- pred_stack[[1]]
  templ_raster[] <- 1
  templ_raster[!complete.cases(pred_grid)] <- NA
  
  pred_grid <- pred_grid[complete.cases(pred_grid),]
  
  # Change categorical variable to factors and others numerical
  for(i in names(pred_grid)){
    if(i %in% categorical){
      if(is.factor(pred_grid[,i]) == FALSE){
        pred_grid[,i] <- as.factor(pred_grid[,i])
      }
    } else {
      if(is.numeric(pred_grid[,i]) == FALSE){
        pred_grid[,i] <- as.numeric(as.character(pred_grid[,i]))
      }
    }
  }
}


##########################################################################################
#### THIS ON EVERYTHING RUNS AUTOMATICALLY
##########################################################################################

#Extract only needed variables
d <- as.data.frame(d)
respvars <- names(d[,respvars])[colSums(d[,respvars] > 0) >= threshold_prevalence]
mod_data <- d[c(respvars, expvars)]

for(i in expvars){
  if(i %in% categorical){
    if(is.factor(mod_data[,i]) == FALSE){
      mod_data[,i] <- as.factor(mod_data[,i])
    }
  } else {
    if(is.numeric(mod_data[,i]) == FALSE){
      mod_data[,i] <- as.numeric(as.character(mod_data[,i]))
    }
  }
}


# Stardardize the response variables if determined
cvr_data <- mod_data
if(stand_method == "stand"){
  for(row in 1:NROW(cvr_data)){
    if(sum(cvr_data[row,respvars]) > 0){
      cvr_data[row,respvars] <- cvr_data[row,respvars]/(rowSums(cvr_data[row,respvars])/100)
    }
  }
}

if(stand_method == "stand_100max"){
  for(row in 1:NROW(cvr_data)){
    if(sum(cvr_data[row,respvars]) > 0){
      cvr_data[row,respvars] <- cvr_data[row,respvars]/ifelse((rowSums(cvr_data[row,respvars])/100)>1,1,
                                                                          (rowSums(cvr_data[row,respvars])/100))
    }
  }
}

if(stand_method == "stand_by_all"){
  for(row in 1:NROW(cvr_data)){
    if(sum(cvr_data[row,respvars]) > 0){
      cvr_data[row,respvars] <- cvr_data[row,respvars]/(cvr_sums[row]/100)
    }
  }
}

if(stand_method == "stand_by_all_100max"){
  for(row in 1:NROW(cvr_data)){
    if(sum(cvr_data[row,respvars]) > 0){
      cvr_data[row,respvars] <- cvr_data[row,respvars]/ifelse(cvr_sums[row]/100 > 1,1,cvr_sums[row]/100)
    }
  }
}


# Transform the response variables if selected
cvr_data_orig <- cvr_data
if(trans_method == "cubic"){
  cvr_data[,respvars] <- apply(cvr_data[,respvars], 2, function(x) ifelse(x == 0, 0, x^(1/3)))
} else {
  if(trans_method == "square"){
    cvr_data[,respvars] <- apply(cvr_data[,respvars], 2, function(x) ifelse(x == 0, 0, x^(1/2)))
  } else {
    if(trans_method == "No"){
      print("Are you 100% sure about not transforming cover values?")
    } else {
      stop("UNKNOWN TRANSFORMATION METHOD")
    }
  }
}

mod_data[,respvars] <- NA
# List of Data Frames for modelling results
pred_df_pred <- list()
for(mod in models){
  pred_df_pred[[mod]] <- mod_data[,respvars]
}
if(ensemble == TRUE){
  pred_df_pred[["ensembleSM"]] <- mod_data[,respvars]
  pred_df_pred[["ensembleWM"]] <- mod_data[,respvars]
}

# Define cross validation methods
ctrl <- trainControl(
  method = cv_method,
  number = cv_number,
  repeats = cv_repeats,
  savePredictions = T
)


# Data frame for evaluation statistics
eval_all <- data.frame()

# List of Data Frames for modelling results
best_model_parameters <- list()

#######################################################################################
# MODELLING LOOP
options(warn=-1)
if(multicore == T){
  cl <- makePSOCKcluster(cores)
  registerDoParallel(cl)
}
start_time <- Sys.time()
if(!dir.exists("./Predictions/")){dir.create("./Predictions/")}
for(i in respvars){
  #i <- "Empetrum_nigrum"
  cat(bold(red(paste(i, Sys.time(),"\n"))))
  
  # MODEL FORMULAS
  formulamodels <- as.formula(paste0(i, " ~ ", paste(expvars, collapse = "+")))
  
  if(prediction == T & "templ_raster" %in% ls()){
    prob_stack <- stack()
  }
  
  # Loop the modelling methods
  models_successed <- c()
  for(mod in models){
    #mod <- "gbm"
    
    # Random Forest
    if(mod == "rf"){
      modstart <- Sys.time()
      print(mod)
      
      set.seed(seed)
      capture.output(
        modm <- train(
          formulamodels,
          data = cvr_data,
          method = "rf",
          preProcess = c("center", "scale"),
          trControl = ctrl,
          tuneGrid=tunegrid_rf, 
          verbose=FALSE,
          metric = "Rsquared"
        ), file='NUL')
      
      print(Sys.time()-modstart)
      
      best_model_parameters[[i]][[mod]] <- modm$bestTune
    }
    # Stochastic Gradient Boosting
    if(mod == "gbm"){
      modstart <- Sys.time()
      print(mod)
      
      set.seed(seed)
      capture.output(
        modm <- train(
          formulamodels,
          data = cvr_data,
          method = "gbm",
          preProcess = c("center", "scale"),
          trControl = ctrl,
          distribution = respfamily,
          verbose=FALSE, 
          tuneGrid=tunegrid_gbm, 
          bag.fraction=0.75,
          metric = "Rsquared"
        ), file='NUL')
      
      print(Sys.time()-modstart)
      
      best_model_parameters[[i]][[mod]] <- modm$bestTune
    }
    
    # Select results with optimal parameters
    if(NCOL(modm$bestTune) == 1){
      modm$pred <- modm$pred[modm$pred[,names(modm$bestTune)[1]] == modm$bestTune[,1],]}
    if(NCOL(modm$bestTune) == 2){
      modm$pred <- modm$pred[modm$pred[,names(modm$bestTune)[1]] == modm$bestTune[,1] &
                               modm$pred[,names(modm$bestTune)[2]] == modm$bestTune[,2],]}
    if(NCOL(modm$bestTune) == 3){
      modm$pred <- modm$pred[modm$pred[,names(modm$bestTune)[1]] == modm$bestTune[,1] &
                               modm$pred[,names(modm$bestTune)[2]] == modm$bestTune[,2] &
                               modm$pred[,names(modm$bestTune)[3]] == modm$bestTune[,3],]}
    if(NCOL(modm$bestTune) == 4){
      modm$pred <- modm$pred[modm$pred[,names(modm$bestTune)[1]] == modm$bestTune[,1] &
                               modm$pred[,names(modm$bestTune)[2]] == modm$bestTune[,2] &
                               modm$pred[,names(modm$bestTune)[3]] == modm$bestTune[,3] &
                               modm$pred[,names(modm$bestTune)[4]] == modm$bestTune[,4],]}
    
    modm$pred <- modm$pred %>% group_by(rowIndex) %>% summarise_all(mean, na.rm = T) %>% as.data.frame()
    
    # Back-transform if needed
    if(trans_method == "cubic"){
      modm$pred$pred <- round(modm$pred$pred^3, 1)
    }
    if(trans_method == "square"){
      modm$pred$pred <- round(modm$pred$pred^2, 1)
    }
    if(trans_method == "No"){
      modm$pred$pred <- round(modm$pred$pred, 1)
    }
    modm$pred$pred <- ifelse(modm$pred$pred < 0, 0, modm$pred$pred)
    
    # R2 value
    if(length(unique(modm$pred$pred)) > 1){
      r2m <- cor(cvr_data_orig[,i], modm$pred$pred)^2
    } else {
      r2m <- 0
    }
    # RMSE
    rmsem <- RMSE(cvr_data_orig[,i], modm$pred$pred)
    
    # PREDICTIONS FOR THE ORIGINAL DATA
    pred_df_pred[[mod]][,i] <- modm$pred[order(modm$pred$rowIndex),"pred"]
    
    # COMBINE EVALUATION METRICS
    evalm <- bind_rows(data.frame(Metric = "R2", 
                                  Value = r2m, mode = "direct", stringsAsFactors = F),
                       data.frame(Metric = "RMSE", 
                                  Value = rmsem, mode = "direct", stringsAsFactors = F))
    
    evalm$model <- modm$method
    evalm$response <- i
    evalm$type <- modm$modelType
    evalm$sampling <- modm$control$sampling$name
    
    # Print evaluation statistics
    cat(paste0(mod, ", R2 direct = ", round(r2m,3), "\n"))
    
    eval_all <- bind_rows(eval_all, evalm)
    
    models_successed <- c(models_successed, mod)
    
    # MAKE PREDICTIONS FOR DIFFERENT SETS OF SCENARIOS
    if(prediction == TRUE){
      for(predid in 1:length(modlist)){
        #predid <- 1
        predtemp <- pred_grid[modlist[[predid]]]
        names(predtemp) <- expvars
        
        ptemp <- predict(modm, predtemp, type = "raw")
        
        templ_raster[!is.na(templ_raster)] <- round(ptemp*10)
        names(templ_raster) <- paste0(names(modlist)[predid], "_", mod)
        
        prob_stack <- stack(prob_stack, templ_raster)
        
        if(only_ensemble_pred == FALSE){
          writeRaster(templ_raster, paste0("Predictions/Predicted_cover_", i, "_", mod, "_",
                                           tag, "_", model_date, "_", names(modlist)[predid], ".tif"),
                      datatype = "INT2U", format = "GTiff", overwrite = T)
        }
      }
    }
  }
  
  # Extract the best fitted parameters from the models
  best_model_parameters[[i]]$models_successful <- models_successed
  
  # Ensemble predictions and evaluations
  if(ensemble == TRUE){
    
    # Weighted mean
    df <- data.frame(id = 1:NROW(pred_df_pred[[models[1]]]))
    for(mod in models_successed){
      df <- bind_cols(df, as.data.frame(pred_df_pred[[mod]][,i])) 
    }
    for(mod in models_successed){
      df <- bind_cols(df, data.frame(kyl = rep(eval_all[eval_all$response == i & 
                                                   eval_all$Metric == ens_metric &
                                                   eval_all$mode == "direct" &
                                                   eval_all$model == mod, "Value"],
                                            times = NROW(df))))
    }
    
    ensdf <- data.frame(id = 1:NROW(cvr_data_orig),
                        obs = cvr_data_orig[,i],
                        pred = apply(df, 1, function(x) weighted.mean(x[2:(length(models_successed)+1)], 
                                                                      x[(length(models_successed)+2):(length(models_successed)*2+1)])))
    
    
    if(length(unique(ensdf$pred)) > 1){
      r2ens <- cor(ensdf$pred, ensdf$obs)^2
    } else {
      r2ens <- 0
    }
    rmseens <- RMSE(ensdf$pred, ensdf$obs)
    
    pred_df_pred[["ensembleWM"]][,i] <- ensdf$pred
    
    evalens <- bind_rows(data.frame(Metric = "R2", 
                                    Value = r2ens, stringsAsFactors = F),
                         data.frame(Metric = "RMSE", 
                                    Value = rmseens, stringsAsFactors = F))
    
    evalens$mode <- "direct"
    evalens$model <- "ensembleWM"
    evalens$response <- i
    evalens$type <- "ensembleWM"
    
    print(paste0("ENSEMBLE, WEIGHTED MEAN, R2 = ", round(evalens[evalens$Metric == "R2", "Value"],3),
                ", RMSE = ", round(evalens[evalens$Metric == "RMSE", "Value"],3)))
    
    eval_all <- bind_rows(eval_all, evalens)
    
    if(prediction == TRUE){
      for(predid in 1:length(modlist)){
        #predid <- 1
        temps <- stack()
        for(mod in models_successed){
          temps <- stack(temps,
                         prob_stack[[names(prob_stack)[grepl(paste0(names(modlist)[predid],"_",mod), names(prob_stack))]]]) 
        }
        
        weights <- c()
        for(mod in unlist(lapply(names(temps), function(x) strsplit(x, "_")[[1]][length(strsplit(x, "_")[[1]])]))){
          
          weights <- c(weights, eval_all[eval_all$response == i & 
                                           eval_all$Metric == ens_metric &
                                           eval_all$mode == "direct" &
                                           eval_all$model == mod, "Value"])
        }
        
        wmr <- round(weighted.mean(temps, weights, na.rm=FALSE))
        wmr[wmr < 0] <- 0
        
        writeRaster(wmr, paste0("Predictions/Predicted_cover_", i, "_", "ensembleWM", "_",
                                tag, "_", model_date, "_", names(modlist)[predid], ".tif"),
                    datatype = "INT2U", format = "GTiff", overwrite = T)
      }
    }
    
    # Simple mean
    ensdf <- data.frame(id = 1:NROW(cvr_data_orig),
                        obs = cvr_data_orig[,i],
                        pred = rowMeans(df[,2:(length(models_successed)+1)]))
    
    if(length(unique(ensdf$pred)) > 1){
      r2ens <- cor(ensdf$pred, ensdf$obs)^2
    } else {
      r2ens <- 0
    }
    rmseens <- RMSE(ensdf$pred, ensdf$obs)
    
    pred_df_pred[["ensembleSM"]][,i] <- ensdf$pred
    
    evalens <- bind_rows(data.frame(Metric = "R2", 
                                    Value = r2ens, stringsAsFactors = F),
                         data.frame(Metric = "RMSE", 
                                    Value = rmseens, stringsAsFactors = F))
    
    evalens$mode <- "direct"
    evalens$model <- "ensembleSM"
    evalens$response <- i
    evalens$type <- "ensembleSM"
    
    print(paste0("ENSEMBLE, SIMPLE MEAN, R2 = ", round(evalens[evalens$Metric == "R2", "Value"],3),
                 ", RMSE = ", round(evalens[evalens$Metric == "RMSE", "Value"],3)))
    
    eval_all <- bind_rows(eval_all, evalens)
    
    if(prediction == TRUE){
      for(predid in 1:length(modlist)){
        
        temps <- stack()
        for(mod in models_successed){
          temps <- stack(temps,
                         prob_stack[[names(prob_stack)[grepl(paste0(names(modlist)[predid],"_",mod), names(prob_stack))]]]) 
        }
        
        wmr <- round(mean(temps, na.rm=FALSE))
        wmr[wmr < 0] <- 0
        
        writeRaster(wmr, paste0("Predictions/Predicted_cover_", i, "_", "ensembleSM", "_",
                                tag, "_", model_date, "_", names(modlist)[predid], ".tif"),
                    datatype = "INT2U", format = "GTiff", overwrite = T)
      }
    }
  }
}
stop_time <- Sys.time()
stop_time-start_time

# Stop clusters
stopCluster(cl)

if(!dir.exists("Modelling_results")){
  dir.create("Modelling_results")
}
write.csv(eval_all, paste0("Modelling_results/model_eval_cvr_", tag, "_", model_date, ".csv"), row.names = F)

eval_all %>% filter(Metric == "R2" & mode == "direct") %>% group_by(model) %>% summarise_all(mean, na.rm = T)
eval_all %>% filter(Metric == "RMSE" & mode == "direct") %>% group_by(model) %>% summarise_all(mean, na.rm = T)

save(best_model_parameters, file = paste0("Modelling_results/best_cvr_model_parameters_", tag, "_", model_date, ".RData"))
save(pred_df_pred, file = paste0("Modelling_results/pred_df_cvr_pred_", tag, "_", model_date, ".RData"))

##########################################################################################################
# Community evaluations
##########################################################################################################

load(file = paste0("Modelling_results/best_cvr_model_parameters_", tag, "_", model_date, ".RData"))
load(paste0("Modelling_results/pred_df_cvr_pred_", tag, "_", model_date, ".RData"))

eval_all <- read.csv(paste0("Modelling_results/model_eval_cvr_", tag, "_", model_date, ".csv"), stringsAsFactors = F, dec = ".")

com_evals <- data.frame()
for(mod in names(pred_df_pred)){
  #mod <- "rf"
  
  sim_pred <- c()
  for(i in 1:NROW(pred_df_pred[[mod]])){
    #i <- 744
    temp <- bind_rows(pred_df_pred[[mod]][i,], cvr_data_orig[i,respvars])
    if(max(rowSums(temp)) == 0){
      sim_pred <- c(sim_pred, 0)
    } else {
      sim_pred <- c(sim_pred, vegdist(temp, method = "bray"))
    }
  }
  
  com_evals <- bind_rows(com_evals,
                         bind_cols(data.frame(model = mod, value = 1-mean(sim_pred), index = "sorensen", 
                                              method = "simple", nspecies = sum(!is.na(colSums(pred_df_pred[[mod]]))), stringsAsFactors = F)))
}

write.csv(com_evals, paste0("Modelling_results/community_eval_", tag, "_", model_date, ".csv"), row.names = F)
com_evals <- read.csv(paste0("Modelling_results/community_eval_", tag, "_", model_date, ".csv"), dec = ".", stringsAsFactors = F)

###################################################################################
# RECONSTRUCTED COMMUNITIES TO COMMUNITY WEIGHTED MEAN TRAITS

list.files(pattern = ".csv")

t <- read.csv("data/Rasti_species_median_traits.csv", dec = ".", stringsAsFactors = F)
t <- t[,c(1,2,6,10,14,18,22,26)]
load(paste0("Modelling_results/pred_df_cvr_pred_", tag, "_", model_date, ".RData"))
load(paste0("Modelling_results/pred_df_cvr_hurdle_", tag, "_", model_date, ".RData"))
d <- cvr_data_orig[,respvars]

# Function for calculating the community weighted means
cvm <- function(trait, com){
  
  trait[,1] <- gsub(" ", "_", trait[,1])
  names(com) <- gsub("\\.", "_", names(com))
  
  m <- matrix(NA, NROW(com), NCOL(trait)-1)
  colnames(m) <- names(trait)[-1]
  for(i in 1:NROW(com)){
    #i <- 1
    vv <- cbind.data.frame(gsub("\\.", "_", names(com)), t(com[i,]))
    names(vv) <- c("species", "cvr")
    
    tv <- full_join(trait, vv, by = "species")
    
    for(ii in 1:NCOL(m)){
      #ii <- 1
      m[i,ii] <- sum(tv[,ii+1]*tv$cvr, na.rm = T)/sum(tv$cvr, na.rm = T)
    }
  }
  return(as.data.frame(m))
}

cwmto <- cvm(t, d)
cwmtm <- cvm(t, pred_df_pred[["ensembleSM"]])
# cwmtm <- cvm(t, pred_df_pred[["ensembleWM"]])
# cwmtm <- cvm(t, pred_df_pred[["rf"]])
# cwmtm <- cvm(t, pred_df_pred[["gbm"]])

# Calculate R2 values between the original and modelled CWM's
allR2 <- c()
for(cnam in names(cwmto)){
  print(cnam)
  #print(round(cor(cwmto[,cnam], cwmtm[,cnam], use = "pairwise.complete.obs")^2,3))
  onecorr <- round(cor(cwmto[,cnam], cwmtm[,cnam], use = "pairwise.complete.obs")^2,3)
  allR2 <- c(allR2, onecorr)
  cat(bold(red(paste0("R2 = ", onecorr, "\n"))))
}
mean(allR2)

names(cwmtm) <- paste0(names(cwmtm), "_mod")

d <- bind_cols(cwmto, cwmtm)
d %>% filter(Height > 0) -> d

# Calculate R2 values between the original and modelled CWM's
# when plots with zero species detected
allR2 <- c()
for(cnam in names(cwmto)){
  print(cnam)
  #print(round(cor(cwmto[,cnam], cwmtm[,cnam], use = "pairwise.complete.obs")^2,3))
  onecorr <- round(cor(d[,cnam], d[,paste0(cnam,"_mod")], use = "pairwise.complete.obs")^2,3)
  allR2 <- c(allR2, onecorr)
  cat(bold(red(paste0("R2 = ", onecorr, "\n"))))
}
mean(allR2)


###########################################################################################################
# SPATIAL AUTOCORRELATION ANALYSES

cwm <- read.csv("data/Rasti_CWM_traits_1325.csv", dec = ".", stringsAsFactors = F)

cwm %>% dplyr::select(siteID, x, y) -> cwm

# Calculate residuals
resids <- cwmto-cwmtm
names(resids) <- paste0(names(resids), "_res")

cwm %>% filter(!is.na(cwmto$Height)) -> cwm

cwm <- bind_cols(cwm, cwmto, resids)

#cwm %>% filter(!is.na(cwmto$Height)) -> cwm

library(ncf)
library(ggplot2)

npoints <- round(NROW(cwm)*0.7) # How many random points
npoints <- 400
runs <- 100 # How many replicates
dist_bins <- 200 # Size of the distance bands
nbins <- 20 # How many distance bands to plot

if(!dir.exists("figures")){
  dir.create("figures")
}
cairo_pdf("figures/Residual_plots.pdf", 9,6, onefile = T)
for(tnam in names(cwmto)){
  print(tnam)
  
  corrs1 <- list()
  corrs2 <- list()
  
  cwm %>% dplyr::select(siteID,x,y,tnam,paste0(tnam,"_res")) %>% 
    filter(complete.cases(.)) -> cwm2
  for(i in 1:runs){
    
    print(i)
    p <- sample(1:NROW(cwm2), npoints, replace = F)
    
    ncf.cor1 <- correlog(x=cwm2[p,"x"], y=cwm2[p,"y"],
                        z=cwm2[p,tnam], increment=dist_bins, resamp=0,
                        latlon = FALSE)
    
    corrs1[[i]] <- ncf.cor1$correlation
    
    ncf.cor2 <- correlog(x=cwm2[p,"x"], y=cwm2[p,"y"],
                         z=cwm2[p,paste0(tnam, "_res")], increment=dist_bins, resamp=0,
                         latlon = FALSE)
    
    corrs2[[i]] <- ncf.cor2$correlation
    
  }
  
  df1 <- bind_cols(lapply(corrs1, function(x) x[1:nbins]))
  df2 <- bind_cols(lapply(corrs2, function(x) x[1:nbins]))
  
  pl <- ggplot()+
    geom_hline(yintercept = -1/(npoints-1), linetype = 2, size = 1)+
    geom_point(aes(x = seq(dist_bins, dist_bins*nbins, length.out = nbins)-20,
                   y = rowMeans(df1)), size = 3)+
    geom_segment(aes(x = seq(dist_bins, dist_bins*nbins, length.out = nbins)-20,
                     y = apply(df1, 1, min), 
                     xend = seq(dist_bins, dist_bins*nbins, length.out = nbins)-20,
                     yend = apply(df1, 1, max)), size = 1.5)+
    geom_point(aes(x = seq(dist_bins, dist_bins*nbins, length.out = nbins)+20,
                   y = rowMeans(df2)), size = 3, color = "orange")+
    geom_segment(aes(x = seq(dist_bins, dist_bins*nbins, length.out = nbins)+20,
                     y = apply(df2, 1, min), 
                     xend = seq(dist_bins, dist_bins*nbins, length.out = nbins)+20,
                     yend = apply(df2, 1, max)), size = 1.5, color = "orange")+
    theme_bw(base_size = 16)+
    ylab("Moran's I ")+
    xlab("Distance (max-of-class)")+
    ggtitle(paste0("SAC for ",tnam))
  
  print(pl)
}
dev.off()

###########################################################################################################
# Raster stacks to CWM figures

# function to calculate CWM traits from raster stacks
cvm_parallel <- function(trait, com){
  
  trait[,1] <- gsub(" ", "_", trait[,1])
  trait[,1] <- gsub("-", "_", trait[,1])
  
  trait <- trait[trait[,1] %in% names(com),]
  com <- com[,trait[,1]]
  
  m <- matrix(NA, NROW(com), NCOL(trait)-1)
  colnames(m) <- names(trait)[-1]
  
  num_splits<-cores*10
  split_testing<-sort(rank(1:nrow(com))%%num_splits)
  
  for(i in 1:NCOL(m)){
    
    tm <- trait[,c(1, i+1)]
    
    m[,i] <- foreach(ii=unique(split_testing),
                         .combine=c) %dopar% {
                           as.numeric(apply(com[split_testing == ii,],1,function(x) as.numeric(sum(tm[,2]*x, na.rm = T)/sum(x, na.rm = T))))
                         }
    
    
    
  }
  
  return(as.data.frame(m))
}

# List the projected cover maps for species
files <- list.files("./Predictions/", pattern = paste0("_ensembleSM_CVR_models_", model_date, "_"), full.names = T)
files <- files[grepl("_cover_", files)]

names(modlist)

# Loop over scenarios and traits for CWM maps
cl <- parallel::makeCluster(4)
doParallel::registerDoParallel(cl)
for(scena in names(modlist)){
  #scena <- names(modlist)[1]
  print(scena)
  tifs <- files[grepl(paste0("_", scena, ".tif"), files)]
  st <- stack(tifs)
  
  names(st) <- unlist(lapply(names(st), function(x) paste(strsplit(x, "_")[[1]][3:4], collapse = "_")))
  names(st) <- gsub("\\.", "_", names(st))
  names(st) <- gsub("_ensembleSM", "", names(st))
  st_grid <- as.data.frame(st)
  st_grid <- st_grid[complete.cases(st_grid),]
  
  st_cwmt <- cvm_parallel(t, st_grid)
  
  for(trait in names(st_cwmt)){
    
    r <- st[[1]]
    r[!is.na(r)] <- st_cwmt[,trait]
    
    writeRaster(r, paste0("Predictions/Predicted_cwm_", trait, "_", "ensembleSM", "_",
                            tag, "_", model_date, "_", scena, ".tif"),
                format = "GTiff", overwrite = T)
    
  }
}
parallel::stopCluster(cl)

